# Generated by Django 2.2.15 on 2020-08-25 14:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0042_rbac_for_tasks'),
        ('ansible', '0021_rename_role_remote'),
        ('galaxy', '0004_rbac'),
    ]

    operations = [
        migrations.CreateModel(
            name='CollectionSyncTask',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('repository', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ansible.AnsibleRepository')),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.Task')),
            ],
            options={
                'ordering': ['-task__finished_at'],
            },
        ),
    ]
