# Generated by Django 2.2.18 on 2021-02-12 17:29

from django.db import migrations
import django_lifecycle.mixins
import galaxy_ng.app.access_control.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('container', '0018_containerdistribution_description'),
        ('galaxy', '0013_partner_engineer_group_migrations'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContainerDistribution',
            fields=[
            ],
            options={
                'proxy': True,
                'default_related_name': '%(app_label)s_%(model_name)s',
                'indexes': [],
                'constraints': [],
            },
            bases=('container.containerdistribution', django_lifecycle.mixins.LifecycleModelMixin, galaxy_ng.app.access_control.mixins.GroupModelPermissionsMixin),
        ),
    ]
