# Generated by Django 2.2.15 on 2020-08-24 16:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0004_rbac'),
        ('galaxy', '0005_merge_synclist_inbound'),
    ]

    operations = [
    ]
