# Generated by Django 2.2.16 on 2020-11-02 14:48

from django.db import migrations


class Migration(migrations.Migration):

    replaces = [('galaxy', '0012_move_collections_by_certification')]

    dependencies = [
        ('galaxy', '0011_collection_import_task_id_fk'),
    ]

    operations = []
