# Generated by Django 3.2.6 on 2021-09-23 16:11

from django.db import migrations, models
import django.db.models.deletion
import galaxy_ng.app.models.contentguard


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0075_rbaccontentguard'),
        ('galaxy', '0023_auto_20210903_1906'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentRedirectContentGuard',
            fields=[
                ('contentguard_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='galaxy_contentredirectcontentguard', serialize=False, to='core.contentguard')),
                ('shared_secret', models.BinaryField(default=galaxy_ng.app.models.contentguard._gen_secret, max_length=32)),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
            },
            bases=('core.contentguard',),
        ),
    ]
